; jmac.def : Declares the module parameters for the DLL.

LIBRARY      "jmac"

EXPORTS
    ; Explicit exports can go here

JNI_OnLoad
JNI_OnUnload
Java_davaguine_jmac_decoder_APEDecompressNative_registerDecoder
Java_davaguine_jmac_decoder_APEDecompressNative_GetData
Java_davaguine_jmac_decoder_APEDecompressNative_Seek
Java_davaguine_jmac_decoder_APEDecompressNative_finalize
